ROMPAGE: EQU	017
RAMPAGE: EQU	016
RAMADDR: EQU	375

CR	EQU	015
PROMPT	EQU	076
DASH	EQU	055
COLON	EQU	072
SPACE	EQU	040
ZERO	EQU	060
EIGHT	EQU	070

INPORT:  EQU 	5
OUTPORT: EQU 	016

;ram layout

RAM:	ORG	RAMPAGE#RAMADDR

	ORG 	ROMPAGE#0
	JMP	CLP	;
NUM:
	CPI	ZERO	; is it less than 0?
	JTS	CLP	; yes, then is an ASCII control char
	NDI	007
	LCA
	LAB		; get current data from buffer
	RLC
	RLC		; shift left 3 bits
	RLC
	NDI	370	; clear low 3 bits
	ORC		; or in 3 new bits
	LBA		; put back in buffer
	JMP	CLP2	; continue without CR
;
; get character from INPUT device and parse
; 260 - 267 = equal octal character - shift into LS 3 bits of reg B
; otherwise look for single character command (scan command table at end of listing)
;
CLP:
	LAI	CR	; send CR
	CAL	OUTPUT	;
	LAI	PROMPT	; send prompt
	CAL	OUTPUT	;	
CLP2:
	CAL	INPUT	; get a byte from serial port
;	ORI	200	; set high bit for compatibility
	CAL	OUTPUT	;
	CPI	EIGHT
	JTS	NUM
	
; table search routine. start at H:L address
; and go to end of page.  each table entry has
; two words: character and lwo adr. if a char is
; found in table Z=0 and jmp to low adr in this
; page 
CMD:
	LDH		; save H reg
	LEL		; save L reg
	LHI	\HB\TTBL ;loop pointers
	LLI	\LB\TTBL ;
CMD2:
	CPM	
	JTZ	CMDFND	; found it
	INL		; 
	INL
	JFZ	CMD2	; not found, get another char

	LHD		; restore HL
	LLE
	JMP	CLP	; get another command
CMDFND:
	INL
	LCM		; get vector
EX:
	LAH		; sav ROMPAGE
	LLI	\LB\RAM	; construct JMP instruction at
	LHI	\HB\RAM	; RAMPAGE
	LMI	104	; insert JMP op code
	INL
	LMC		; save vector
	INL
	LMA		; high address was in H before
	LHD
	LLE
	JMP	RAM     ; execute it
;
; Command X - execute routine at address HL
CX:
	LCL		; H is good to go, but L needs to be moved to C
	JMP	EX	
;
; Command H - copy BUFFER to H
CH:	LHB		; put B into H
	JMP	DHL	; and display current value of HL
;
; Command L - copy BUFFER to L
CL:	LLB		; Buffer B into L
	JMP	DHL	; DUMP PTR and CONTENTS
;
; Command W - write BUFFER through HL to memory
; auto increment and display next location
CW:	LMB		; write B at address in memory
;
; Command I - increment HL 
CI:	INL		; increment L
	JFZ 	DHL	; no wrap, display memory	
	INH		; if L=0 increment H
	JMP 	DHL	; continue to display memory
;
; Command D - decrement HL
CD:	LAL		; save in A, (use to check for wrap)
	DCL		; decrement L
	NDA		; check for wrap
	JFZ 	DHL	; no wrap, now display memory
	DCH		; wrap, decrement H
;
; Command D - current memory contents  to serial port
CC:	
DHL:
	LAI	SPACE	; insert space
	CAL	OUTPUT	;
	LAH
	CAL	POCTAL	; display H
	LAI	DASH	; dash between high and low
	CAL	OUTPUT	;
	LAL		; display L
	CAL	POCTAL	; 
	LAI	COLON	; colon between address and data
	CAL	OUTPUT
	LAM		; display contents
	CAL	POCTAL	; print octal
	JMP	CLP	; get next command
;
; print octal number
; call this routine, don't jump to it
;
POCTAL:
	LEA
	RLC
	RLC
	NDI	003
	CAL	POCTAL2	; display MSB 2 bits
	LAE
	RAR
	RAR
	RAR
	CAL	POCTAL2	; display middle 3 bits
	LAE		; display LSB 3 bits
POCTAL2:
	NDI	007	; mask 3 bits
	ORI	ZERO	; make ASCII
	JMP 	OUTPUT	; send it
;
; Command B - run serial boot loader - when load is done, user must restart monitor
; 		reads a serial stream of binary data into memory start at current
;		value of HL
CBT:
	CAL	INPUT   ; get byte from serial port
	LMA 		; store byte
	INL 		; increment pointer
	JFZ CBT	 	; wrap?
	INH 		; increment MSB
	JMP CBT
;
; HERE IS THE USER DEFINED CHARACTER INPUT TO READ FROM SERIAL PORT
; destroys C & D
; returns character in A
INPUT:
        INP INPORT
        NDA                     ; 5
        JTS INPUT               ; 9/11
; found - delay 1 1/2 bit times
        LCI     5             	; 8  209 decimal
	CAL	TIMER3		; delay 1.5 bit times for start bit
	LDI	8		; 8 data bits
;grab 8 bits
IN2:
	CAL	DATAIN		; get bit
	DCD
	JFZ	IN2		; repeat
; since we are echoing character, no need to
; wait for stop bit here
;
; result is in a, echo it
	RET
;
; get 1 bit
; MSB of port contains input stream
; read and move to carry
; shift into MS bit position
; and maintain in B register
DATAIN:
        LCA                     ; 5
        INP INPORT              ; 8
        RAL                     ; 5
        LAC                     ; 5
        RAR                     ; 5
; delay 1 bit time (used by both input and output)
; destroys B
TIMER4:
        LCI     2             	; 8  2 decimal
TIMER3:                         ;
        DCC                     ; 5
        JFZ TIMER3              ; 9/11
        LAA
        RET
;
; HERE IS THE USER DEFINED PRINT ROUTINE FOR A SERIAL PORT
; input:A
; destroys C & D
;
OUTPUT:  NDA
        RAL
        OUT 	OUTPORT		; start bit
        RAR
	LCI	3	
        CAL 	TIMER3
	LDI	8		; 8 data bits
OUT2:
        CAL 	DATAOUT		; next data bit
	DCD
	JFZ 	OUT2

        LCA
        LAI 	001		; stop bit
        OUT 	OUTPORT
        LAC
        JMP 	TIMER4

DATAOUT: OUT 	OUTPORT
        RRC
        CAL 	TIMER4
	RET
;
; this table has octal charctets, control
; characters and addres of control subroutine
; note that first tabel entry is also ret
; above, I E overlap is OK
	ORG	ROMPAGE#360
TTBL:
	DATA	'L'
	DATA	\LB\CL	; put buffer in low adr
	DATA	'H'	; H
	DATA	\LB\CH	; put buffer in high adr
	DATA	'I'	; I
	DATA	\LB\CI	; increment adr pointer
	DATA	'W'	; M
	DATA	\LB\CW	; write buffer in memory
	DATA	'X'	; X
	DATA	\LB\CX	; execute loaded program
	DATA	'B'	; O
	DATA	\LB\CBT  ; run serial boot loader to load memory
	DATA	'D'	; D
	DATA	\LB\CD	; decrement address pointer
	DATA	'C'	; C
	DATA	\LB\CC	; display memory on panel
